
-- DToX, 2014-2015
-- Licensed under the same terms as Lua itself.

-- ******************
-- * Text functions *
-- ******************

-- Functions: getSize, split
--[[
getSize
split
--]]

--[[ Examples
Calculate width (limit to ~1-2 symbol on the line) and height, return size
   local W, H = Text:getSize( "1234", Turbine.UI.Lotro.Font.Verdana14 );

Limit width to 50, calculate height, return size
   local W, H = Text:getSize( "1234", Turbine.UI.Lotro.Font.Verdana14, 50 );

Calculate width and height, return size. Supports text with \n.
!! Doesn't limit maximum width to the screen's width.
   local W, H = Text:getSize( "1234", Turbine.UI.Lotro.Font.Verdana14, "Auto" );
--]]



local wl = Turbine.Shell.WriteLine;

Text = {};
Text.Step = {
   Height = 1;
   Width = 1;
   Maximum = 100000;
};


-- Returns width or height of the text
local function getSideSize( objScroll, sideName )
   local i = 0;
   local sideSize;
   local stepSizeH = Text.Step.Height;
   local stepSizeW = Text.Step.Width;
   local maxSteps = Text.Step.Maximum;

   while ( objScroll:IsVisible() ) do
      i = i + 1;
      if ( sideName == "H" ) then
         sideSize = stepSizeH * i;
         Text.Label:SetHeight( sideSize );
      elseif ( sideName == "W" ) then
         sideSize = stepSizeW * i;
         Text.Label:SetWidth( sideSize );
      end

      -- Just in case. To prevent infinite loop
      i = i + 1;
      if ( i > maxSteps ) then
         wl( ">> Error. Text.getSideSize (" .. sideName .. "): " .. i
         .. " steps, possible infinite loop.\nOr a very long text..." );
         return nil;
      end
   end

   return sideSize;
end

-- Resets and sets text's values, returns width and height of a single line
local function getLineSize( text, font, widthOrAuto, ignoreHeight )
   local W;
   local H;
   local bWidth = false;

   local label = Turbine.UI.Label();
   label:SetSize( 0, 0 );
   label:SetMarkupEnabled( true );

   -- Scroll. For measuring text's height
   local vScroll = Turbine.UI.Lotro.ScrollBar();
   vScroll:SetParent( label );
   label:SetVerticalScrollBar( vScroll );

   -- Scroll. For measuring text's width
   local hScroll = Turbine.UI.Lotro.ScrollBar();
   hScroll:SetParent( label );
   label:SetHorizontalScrollBar( hScroll );


   -- Determine mode: fixed width or not
   if ( type( widthOrAuto ) == "number" ) then
      bWidth = true;
      W = widthOrAuto;
      label:SetWidth( W );
   end

   -- If width argument is present, but not a number.
   -- Determine multiline or not
   label:SetMultiline( true );
   if ( widthOrAuto and not bWidth ) then
      label:SetMultiline( false );
   end


   label:SetFont( font );
   label:SetText( text ); -- Have to be after scrolls
   Text.Label = label;

   -- Calculate width
   function hScroll.VisibleChanged()
      if ( not bWidth ) then
         W = getSideSize( hScroll, "W" );
      end
   end
   hScroll.VisibleChanged();

   -- Calculate height. Don't if argument ignoreHeight is present
   if ( not ignoreHeight ) then
      function vScroll.VisibleChanged()
         H = getSideSize( vScroll, "H" );
      end
      vScroll.VisibleChanged();
   end

   return W, H;
end

-- Calculates and returns text's width and height
Text.getSize = function( self, text, font, widthOrAuto )
   if ( not text or text == "" ) then -- Why would you...
      wl( ">> Error. Text.getSize. Please, set the text." );
      return;
   end

   local W;
   local H;
   local t = type( widthOrAuto );

   -- Width is not a number and text contains new lines
   if ( t ~= "number" and text:find( "\n" ) ) then
      -- Find maximum width
      local strings = Text.split( text, "\n" );
      local maxWidth = 0;
      for i, str in pairs( strings ) do
         width = getLineSize( str, font, widthOrAuto, "IgnoreHeight" );
         if ( width > maxWidth ) then
            maxWidth = width;
         end
      end

      W, H = getLineSize( text, font, maxWidth );
   else
      W, H = getLineSize( text, font, widthOrAuto );
   end

   return W, H;
end

-- Returns first split before separator and rest of the string or original string
local function getNextSplit( str, separator )
   local pos = str:find( separator );
   if ( pos ) then
      local split = str:sub( 0, pos - 1 );
      local newStr = str:sub( pos + 1 );
      return split, newStr;
   else
      return str;
   end
end

-- Splits string into an array of substrings using separator.
-- Ignores empty substrings
Text.split = function( str, separator )
   local newStr = str;
   local splits = {};
   local split;
   local i = 0;
   while ( newStr ) do
      split, newStr = getNextSplit( newStr, separator );
      if ( split and split ~= "" ) then
         i = i + 1;
         splits[i] = split;
      end
if ( i > 10000 ) then return; end -- Just in case
   end
   
   return splits;
end





_G.DToX_Lib = _G.DToX_Lib or {};
_G.DToX_Lib.Text = Text;
